/*****************************************************************************
  BH1749NUC.h

 Copyright (c) 2022 ROHM Co.,Ltd.

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
******************************************************************************/
#ifndef BH1749NUC_H_
#define BH1749NUC_H_

#ifndef float32
#define float32                                 float
#endif

// Error Number
#define BH1749NUC_COMM_OK                       (0)
#define BH1749NUC_COMM_ERROR                    (-1)
#define BH1749NUC_ID_ERROR                      (-2)
// Slave Address
#define BH1749NUC_SLAVE_ADDRESS_38              (0x38)
#define BH1749NUC_SLAVE_ADDRESS_39              (0x39)
#define BH1749NUC_ADDR_HIGH                     (true)
#define BH1749NUC_ADDR_LOW                      (false)
// Register Address
#define BH1749NUC_SYSTEM_CONTROL                (0x40)
#define BH1749NUC_MODE_CONTROL1                 (0x41)
#define BH1749NUC_MODE_CONTROL2                 (0x42)
#define BH1749NUC_RED_DATA_LSB                  (0x50)
#define BH1749NUC_MANUFACTURER_ID               (0x92)
// Register Parameter
#define BH1749NUC_CNTL1_MEAS_MODE_120MS         (2 << 0)
#define BH1749NUC_CNTL1_RGB_GAIN_X1             (1 << 3)
#define BH1749NUC_CNTL1_IR_GAIN_X1              (1 << 5)
#define BH1749NUC_CNTL2_RGB_EN                  (1 << 4)
// Register Value(write)
#define BH1749NUC_MODE_CONTROL1_VAL             (BH1749NUC_CNTL1_MEAS_MODE_120MS | BH1749NUC_CNTL1_RGB_GAIN_X1 | BH1749NUC_CNTL1_IR_GAIN_X1)
// Register Value(read)
#define BH1749NUC_PART_ID_VAL                   (0x0D)
#define BH1749NUC_MANUFACTURER_ID_VAL           (0xE0)
// Mask
#define BH1749NUC_PART_ID_MASK                  (0x3F)
#define BH1749NUC_MANUFACTURER_ID_MASK          (0xE0)
// Version
#define BH1749NUC_DRIVER_VERSION                (1.1F)
// Others
#define BH1749NUC_COLOR_NUMBER                  (5)
#define BH1749NUC_RED_DATA_SIZE                 (12)

class BH1749NUC
{
    public:
        BH1749NUC(bool addr_pin);
        ~BH1749NUC();
        int32_t init(void);
        int32_t start(void);
        int32_t stop(void);
        int32_t get_val(uint8_t *raw);
        int32_t get_val(uint16_t *data);
        float32 get_version(void);

    private:
        uint8_t Address;

        int32_t init_check_val(void);
        int32_t init_setting_ic(void);
        int32_t write(uint8_t reg, uint8_t *data, int32_t size);
        int32_t read(uint8_t reg, uint8_t *data, int32_t size);
};

#endif // BH1749NUC_H_